//-------------------------------------------------------------------------------------
//	dummy µtracer
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//	Phil le   13/01/2013
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//	includes
//-------------------------------------------------------------------------------------
#define classe_e
#include"main.h"
#undef classe_e
//-------------------------------------------------------------------------------------
//gestion des interruptions
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//reception d'un message en provenance de la liaison RS232
//-------------------------------------------------------------------------------------
ISR(USART_RXC_vect)
{
	unsigned char cmde;	//commande reçue
	float valeur;		//valeur reçue

	//reception du mot de commande
	cmde=rs232_rx_8();

	//la cmde est elle dans le dictionnaire ?
	// si non, on fait pointer sur une commande "nulle" qui ne fait rien
	if (cmde>ncom)
	{
		cmde=0;
	}

	if (cmde<0)
	{
		cmde=0;
	}

	//reception du flottant
	valeur=rs232_rx_32();

	//juste pour chez moi, j'affiche la commande sur un reseau de leds branché sur le port C
	PORTC=cmde;

	//appel de la routine pointée par la commande
	pf[cmde](valeur);
}
//-------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------
//routines
//-------------------------------------------------------------------------------------
//courant du tube 12ax7
uint8_t courant(float va,float vg,float *ia)
{
	//on calcule ia d'après un modèle ultra simplifié
	//le courant de grille sera nul
	//on bride le courant de plaque entre 0 et 5mA
	//en cas de dépassement le code retour est 1 pour ia>5mA et 2 pour ia<0

	uint8_t res;

	float mu;
	float ex;
	float kg1;
	float kp;
	float kvb;

	mu  = 107.282;
	ex  = 0.900255;
	kg1 = 870.263;
	kp  = 875.025;
	kvb = 6112.42;

	//adaptation du modele
	vg=vg+0.5;

	*ia = va/kp*log(1+exp(kp*(1/mu+vg/sqrt(kvb+va*va)))) * 1.5;


	//limitations du modèle
	res=0;

	if (*ia>3.5)
	{
		*ia=3.5;
		res=1;
	}

	if (*ia<0)
	{
		*ia=0;
		res=2;
	}

	return res;
}

//-------------------------------------------------------------------------------------
//init
void init(void)
{
	//initialisation du tableau de pointeur de fonction

	int i;

	//juste pour chez moi, j'affiche la commande sur un reseau de leds branché sur le port C
	DDRC=255;

	i=0;

	pf[i] = fonc_void; i++;	//fonction "nulle"

	pf[i] = set_VA_max; i++;
	pf[i] = set_VG2_max; i++;
	pf[i] = set_VG1_max; i++;
	pf[i] = set_VFIL_max; i++;
	pf[i] = set_IA_max; i++;
	pf[i] = set_IG2_max; i++;
	pf[i] = set_IG1_max; i++;
	pf[i] = set_PWA_max; i++;
	pf[i] = set_PWG2_max; i++;
	pf[i] = set_PWG1_max; i++;
	pf[i] = set_VG2_min; i++;
	pf[i] = set_VG1_min; i++;
	pf[i] = set_VFIL_min; i++;
	pf[i] = set_VA; i++;
	pf[i] = set_VG2; i++;
	pf[i] = set_VG1; i++;
	pf[i] = set_VFIL; i++;

	pf[i] = mesure;
}
//-------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------
//programme principal
//-------------------------------------------------------------------------------------
int main(void)
{
	//init
	init();

	//init rs232 en bidirectionnel à 115200 bauds
	rs232_init(16,1);

	//boucle infinie
	while(1)
	{
	}

	return 0;
}

//-------------------------------------------------------------------------------------
//routines
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//initialisation de la transmission
//-------------------------------------------------------------------------------------
void rs232_init(unsigned char baud,unsigned char isr_rx)
{
	cli();

	//reglage de la vitesse
 	UBRRH=(unsigned char)(baud>>8);
	UBRRL=(unsigned char)baud;

	//autorisation transmission reception
	UCSRB=(1<<RXEN)|(1<<TXEN);

	//reglage protocole (8 bits de donnees, 2 bits de stop)
	UCSRC=(1<<URSEL)|(1<<USBS)|(3<<UCSZ0);

	//reglage mode asynchrone
 	UCSRC=UCSRC | 64;

 	//reglage double debit
	UCSRA=UCSRA | 2;

	//autorisation interruption sur reception
	if (isr_rx!=0)
	{
		UCSRB = UCSRB | 128;
	}

	sei();
}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//reception d'un mot de 8 bits
//-------------------------------------------------------------------------------------
unsigned char rs232_rx_8(void)
{
	//attente de reception
	while (!(UCSRA & (1<<RXC)))
	{
	}

	//reception et retour
	return UDR;
}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//emission d'un mot de 8 bits
//-------------------------------------------------------------------------------------
void rs232_tx_8(unsigned char v)
{
	//cli();

	//attente buffer emission vide
	while (!(UCSRA & (1<<UDRE)))
	{
	}

	//envoi
	UDR=v;

	//sei();
}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//reception d'un mot de 16 bits
//-------------------------------------------------------------------------------------
uint16_t rs232_rx_16(void)
{
	volatile uint16_t res;
	volatile unsigned char msb;
	volatile unsigned char lsb;

	msb=rs232_rx_8();
	lsb=rs232_rx_8();

	res=msb*256+lsb;

	return res;
}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//emission d'un mot de 16 bits
//-------------------------------------------------------------------------------------
void rs232_tx_16(uint16_t v)
{
	volatile unsigned char msb;
	volatile unsigned char lsb;

	msb=v/256;
	lsb=v-msb*256;

	rs232_tx_8(msb);
	rs232_tx_8(lsb);
}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//emission d'un mot de 32 bits
//-------------------------------------------------------------------------------------
//reception 32
float rs232_rx_32(void)
{
	volatile float *res;
	volatile uint8_t val[4];
	volatile uint8_t i;

	for (i=0;i<4;i++)
	{
		val[i]=rs232_rx_8();
	}

	res=(float*)val;

	return *res;
}
//-------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------
//emission d'un mot de 32 bits
//-------------------------------------------------------------------------------------
//emission 32
void rs232_tx_32(float v)
{
	volatile uint8_t *val;
	volatile uint8_t i;

	val=(uint8_t*)&v;

	for (i=0;i<4;i++)
	{
		rs232_tx_8(*val);
		val++;
	}
}
//-------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------
//fonction nulle
//-------------------------------------------------------------------------------------
void fonc_void(float v)
{
	//on retourne juste un code 99 qui indique que la commande passée
	//n'a pas été interprétée

	rs232_tx_8(99);
}

//-------------------------------------------------------------------------------------
//fonctions set_xx_max et set_xx_min
//
//Ces fonction ne font que memoriser les valeurs limites à ne pas
//depasser pendant la manip
//
//-------------------------------------------------------------------------------------
void set_VA_max(float v)
{
	VA_max=v;
	rs232_tx_8(0);
}

//-------------------------------------------------------------------------------------
void set_VG2_max(float v)
{
	VG2_max=v;
	rs232_tx_8(0);
}

//-------------------------------------------------------------------------------------
void set_VG1_max(float v)
{
	VG1_max=v;
	rs232_tx_8(0);
}

//-------------------------------------------------------------------------------------
void set_VFIL_max(float v)
{
	VFIL_max=v;
	rs232_tx_8(0);
}

//-------------------------------------------------------------------------------------
void set_IA_max(float v)
{
	IA_max=v;
	rs232_tx_8(0);
}

//-------------------------------------------------------------------------------------
void set_IG2_max(float v)
{
	IG2_max=v;
	rs232_tx_8(0);
}

//-------------------------------------------------------------------------------------
void set_IG1_max(float v)
{
	IG1_max=v;
	rs232_tx_8(0);
}

//-------------------------------------------------------------------------------------
void set_PWA_max(float v)
{
	PWA_max=v;
	rs232_tx_8(0);
}

//-------------------------------------------------------------------------------------
void set_PWG2_max(float v)
{
	PWG2_max=v;
	rs232_tx_8(0);
}

//-------------------------------------------------------------------------------------
void set_PWG1_max(float v)
{
	PWG1_max=v;
	rs232_tx_8(0);
}

//-------------------------------------------------------------------------------------
void set_VG2_min(float v)
{
	VG2_min=v;
	rs232_tx_8(0);
}

//-------------------------------------------------------------------------------------
void set_VG1_min(float v)
{
	VG1_min=v;
	rs232_tx_8(0);
}

//-------------------------------------------------------------------------------------
void set_VFIL_min(float v)
{
	VFIL_min=v;
	rs232_tx_8(0);
}

//-------------------------------------------------------------------------------------
//fonctions set_xx
//
//Ces fonctions fixent les tensions à appliquer avant chaque mesure
//Elles ont donc une action sur le hard
//
//-------------------------------------------------------------------------------------
void set_VA(float v)
{
	uint8_t ret;

	VA=v;
	ret=0;

	//limitations
	if (VA>VA_max)
	{
		ret=1;
	}

	//...
	//...


/*
	action sur le booster à definir
	par exemple:

	if (ret==0)
	{
		tension_ok=0;
		nb_tentatives=0;

		while (tension_ok==0 && nb_tentative<nb_tentative_max)
		{
			tension_ok=1;
			ret=0;

			ecart=mesure_can(voie_va)-VA;
			nb_tentative++;

			if (ecart>VA+seuil)
			{
				booster_VA_diminue;
				tension_ok=0;
				ret=2;
			}

			if (ecart<VA-seuil)
			{
				booster_VA_augmente;
				tension_ok=0;
				ret=1;
			}

		}

		if (nb_tentative>=nb_tentative_max)
		{
			ret=3;
		}
	}
*/

	rs232_tx_8(ret);
}

//-------------------------------------------------------------------------------------
void set_VG2(float v)
{
	VG2=v;

	//action sur le booster
	//idem reglage de VA

	rs232_tx_8(0);
}

//-------------------------------------------------------------------------------------
void set_VG1(float v)
{
	VG1=v;

	//action sur le booster
	//idem reglage de VA

	rs232_tx_8(0);
}

//-------------------------------------------------------------------------------------
void set_VFIL(float v)
{
	VFIL=v;

	//action sur le booster
	//idem reglage de VA

	rs232_tx_8(0);
}


//-------------------------------------------------------------------------------------
//fonction mesure
//-------------------------------------------------------------------------------------
void mesure(float v)
{
/*
	operation de mesure à definir
	par exemple:

	enclenche_HT

	//mesure unitaire pour voir si on ne dépasse pas les mim max
	va=mesure_va();
	vg1=mesure_vg1();
	vg2=mesure_vg2();
	vfil=mesure_vfil();
	ia=mesure_ia();
	ig1=mesure_ig1();
	ig2=mesure_ig2();

	arret_HT

	if (va<va_min)
	{
		ret=1;
	}

	if (va>va_max)
	{
		ret=2;
	}

	etc... avec vg1 vg2 et vfil

	if (ia>ia_max)
	{
		ret=yy;
	}

	etc... avec ig1 ig2

	pwa=ia*1000*va;

	if (pwa>pwa_max)
	{
		ret=xx;
	}

	etc... avec pwg1 pwg2


	if (ret==0)
	{
		//pas de dépassement de limites donc on fait les moyennes

		//init
		va_moy=0;
		...
		...
		...

		enclenche_HT

		for (i=0;i<nmoy;i++)
		{
			va_moy=va_moy+mesure_va();
			...
			...
			...
		}

		arret_HT

		//calcul moy
		va_moy=va_moy/nmoy;
		...
		...
		...


	}

	//renvoi des valeurs
	rs232_tx_8(ret);
	rs232_tx_32(va_moy);
	...
	...
	...
*/

	//c'est là que je place mon dummy 12AX7
	//encapsulé dans une boucle

	uint8_t ret;
	uint8_t i;
	uint8_t nmoy;
	float va_moy;
	float vg1_moy;
	float vg2_moy;
	float vfil_moy;
	float ia_moy;
	float ig1_moy;
	float ig2_moy;
	uint8_t dep_VA;
	uint8_t dep_VG1;
	uint8_t dep_VG2;
	uint8_t dep_IA;
	uint8_t dep_IG1;
	uint8_t dep_IG2;
	uint8_t dep_PWA;
	uint8_t dep_PWG1;
	uint8_t dep_PWG2;
	float mes_courant;
	float mes_puissance;

	//recuperation du nombre de moyennes
	nmoy=(uint8_t)v;

	//limitations du nombre de moyennes
	if (nmoy<=0)
	{
		nmoy=1;
	}

	if (nmoy>100)
	{
		nmoy=100;
	}

	//init
	va_moy=0;
	vg1_moy=0;
	vg2_moy=0;
	vfil_moy=0;
	ia_moy=0;
	ig1_moy=0;
	ig2_moy=0;



	//verification si depassement des valeurs min max
	dep_VA=0;
	dep_VG1=0;
	dep_VG2=0;
	dep_IA=0;
	dep_IG1=0;
	dep_IG2=0;
	dep_PWA=0;
	dep_PWG1=0;
	dep_PWG2=0;

	ret=0;

	if (VA>VA_max)
	{
		dep_VA=2;
		ret=1;
	}

	if (VG2>VG2_max)
	{
		dep_VG2=2;
		ret=1;
	}

	if (VG2<VG2_min)
	{
		dep_VG2=1;
		ret=1;
	}

	if (VG1>VG1_max)
	{
		dep_VG1=2;
		ret=1;
	}

	if (VG1<VG1_min)
	{
		dep_VG1=1;
		ret=1;
	}

	//si ret!=0 ---> depassement donc pas de mesure
	if (ret==0)
	{
		//mesures unitaires des courants pour verifier des depassements

		//mesure ia
		ret=courant(VA,VG1,&mes_courant);
		mes_puissance=VA*mes_courant/1000;

		if (mes_courant>IA_max)
		{
			dep_IA=1;
			ret=1;
		}

		if (mes_puissance>PWA_max)
		{
			dep_PWA=1;
			ret=1;
		}

		//mesure ig1
		mes_courant=0;	//pour le dummy 12AX7
		mes_puissance=VG1*mes_courant/1000;

		if (mes_courant>IG1_max)
		{
			dep_IG1=1;
			ret=1;
		}

		if (mes_puissance>PWG1_max)
		{
			dep_PWG1=1;
			ret=1;
		}

		//mesure ig2
		mes_courant=0;	//pour le dummy 12AX7
		mes_puissance=VG2*mes_courant/1000;

		if (mes_courant>IG2_max)
		{
			dep_IG2=1;
			ret=1;
		}

		if (mes_puissance>PWG2_max)
		{
			dep_PWG2=1;
			ret=1;
		}

		//si ret!=0 ---> depassement donc pas de moyenne des mesures
		if (ret==0)
		{
			//mesures
			for (i=0;i<nmoy;i++)
			{
				va_moy=va_moy+VA;		//pour le dummy 12AX7
				vg2_moy=vg2_moy+VG2;	//pour le dummy 12AX7
				vg1_moy=vg1_moy+VG1;	//pour le dummy 12AX7

				ret=courant(VA,VG1,&mes_courant);//pour le dummy 12AX7
				ia_moy=ia_moy+mes_courant;

				ig2_moy=ig2_moy+0;	//pour le dummy 12AX7
				ig1_moy=ig1_moy+0;	//pour le dummy 12AX7
			}

			//moyennes
			va_moy=va_moy/nmoy;
			vg2_moy=vg2_moy/nmoy;
			vg1_moy=vg1_moy/nmoy;
			ia_moy=ia_moy/nmoy;
			ig2_moy=ig2_moy/nmoy;
			ig1_moy=ig1_moy/nmoy;
		}
		else
		{
			//il y a une demande de mesure avec des parametres valides
			//mais qui engendrent des courants ou puissances hors domaine
			//le vecteur de retour indique quelle variable depasse
			//le code retour donne des indication sur les puissance dépassées
			// 2=pwa 4=pwg1 8=pwg2  (code 11 --> pwamax et pwg2max)

			va_moy=dep_VA;
			vg2_moy=dep_VG2;
			vg1_moy=dep_VG1;
			ia_moy=dep_IA;
			ig2_moy=dep_IG2;
			ig1_moy=dep_IG1;

			ret=dep_PWA*2+dep_PWG1*4+dep_PWG2*8+1;
		}
	}
	else
	{
		//il y a une demande de mesure avec des parametres qui sortent du domaine valide
		//le code retour vaut 1 et le vecteur de retour indique quelle variable depasse
		va_moy=dep_VA;
		vg1_moy=dep_VG1;
		vg2_moy=dep_VG2;
	}

	rs232_tx_8(ret);
	rs232_tx_32(va_moy);	//va
	rs232_tx_32(vg2_moy);	//vg2
	rs232_tx_32(vg1_moy);	//vg1
	rs232_tx_32(ia_moy);	//ia
	rs232_tx_32(ig2_moy);	//ig2
	rs232_tx_32(ig1_moy);	//ig1

}








